@@ -31,30 +31,17 @@ class S3VFS(apsw.VFS):
         return S3VFSFile(name, flags, self._bucket, self._block_size)
 
     def serialize(self, key_prefix):
-        bytes_so_far = 0
+        for obj in self._bucket.objects.filter(Prefix=key_prefix + '/'):
+            yield from obj.get()['Body'].iter_chunks()
 
-        for i, obj in enumerate(self._bucket.objects.filter(Prefix=key_prefix + '/')):
-            block_bytes = obj.get()['Body'].read()
-
-            if i == 0:
-                page_size, = struct.Struct('>H').unpack(block_bytes[16:18])
-                page_size = 65536 if page_size == 1 else page_size
-                num_pages, = struct.Struct('>L').unpack(block_bytes[28:32])
-                bytes_expected = page_size * num_pages
-
-            to_yield = min(len(block_bytes), bytes_expected - bytes_so_far)
-            yield block_bytes[:to_yield]
-            bytes_so_far += to_yield
 
 class S3VFSFile:
     def __init__(self, name, flags, bucket, block_size):
-        if isinstance(name, apsw.URIFilename):
-            self._key_prefix = name.filename()
-        else:
-            self._key_prefix = name
+        self._key_prefix = \
+            self._key_prefix = name.filename() if isinstance(name, apsw.URIFilename) else \
+            name
         self._bucket = bucket
         self._block_size = block_size
-        self._empty_block_bytes = bytes(self._block_size)
 
     def _blocks(self, offset, amount):
         while amount > 0:
@@ -72,10 +59,8 @@ class S3VFSFile:
         try:
             block_bytes = self._block_object(block).get()["Body"].read()
         except self._bucket.meta.client.exceptions.NoSuchKey as e:
-            block_bytes = self._empty_block_bytes
+            block_bytes = b''
 
-        assert type(block_bytes) is bytes
-        assert len(block_bytes) == self._block_size
         return block_bytes
 
     def xRead(self, amount, offset):
@@ -108,33 +93,31 @@ class S3VFSFile:
         return True
 
     def xTruncate(self, newsize):
-        num_blocks = (newsize + self._block_size - 1) // self._block_size
+        total = 0
+
         for obj in self._bucket.objects.filter(Prefix=self._key_prefix + "/"):
-            block = int(obj.key[-10:])
-            if block >= num_blocks:
+            total += obj.size
+            to_keep = max(obj.size - total + newsize, 0)
+
+            if to_keep == 0:
                 obj.delete()
+            elif to_keep < obj.size:
+                obj.put(Body=obj.get()['Body'].read()[:to_keep])
+
         return True
 
     def xWrite(self, data, offset):
         data_offset = 0
         for block, start, write in self._blocks(offset, len(data)):
 
-            assert write <= len(data)
+            data_to_write = data[data_offset:data_offset+write]
 
-            if write == len(data) == self._block_size:
-                # No need to fetch the original bytes, since we completely replace them
-                new_block_bytes = data
-            else:
+            if start != 0 or len(data_to_write) != self._block_size:
                 original_block_bytes = self._block_bytes(block)
-                new_block_bytes = b"".join([
-                    original_block_bytes[0:start],
-                    data[data_offset:data_offset+write],
-                    original_block_bytes[start+write:],
-                ])
+                data_to_write = \
+                    original_block_bytes[0:start] + \
+                    data_to_write + \
+                    original_block_bytes[start+write:]
+
             data_offset += write
-
-            assert len(new_block_bytes) == self._block_size
-
-            self._block_object(block).put(
-                Body=new_block_bytes,
-            )
+            self._block_object(block).put(Body=data_to_write)
